
package ch.ethz.ssh2.channel;

import java.io.IOException;
import java.io.InputStream;

/**
 * ChannelInputStream.
 * 
 * @author Christian Plattner, plattner@inf.ethz.ch
 * @version $Id: ChannelInputStream.java,v 1.2 2005/06/13 10:51:52 cplattne Exp $
 */
public final class ChannelInputStream extends InputStream
{
	Channel c;

	boolean isClosed = false;
	boolean isEOF = false;
	boolean extendedFlag = false;

	byte[] buffer = new byte[Channel.CHANNEL_BUFFER_SIZE];
	int bufferpos = 0;
	int bufferlen = 0;

	ChannelInputStream(Channel c, boolean isExtended)
	{
		this.c = c;
		this.extendedFlag = isExtended;
	}

	private void fillBuffer() throws IOException
	{
		bufferpos = 0;
		bufferlen = 0;

		if (isEOF == true)
			return;

		bufferlen = c.cm.getChannelData(c, extendedFlag, buffer);

		if (bufferlen == 0)
		{
			isEOF = true;
			return;
		}
	}

	public int available() throws IOException
	{
		if (bufferlen > 0)
			return bufferlen;

		if (isEOF)
			return 0;

		int avail = c.cm.getAvailable(c, extendedFlag);

		return (avail > 0) ? avail : 0;
	}

	public void close() throws IOException
	{
		isClosed = true;
	}

	public int read(byte[] b, int off, int len) throws IOException
	{
		if (bufferlen == 0)
		{
			fillBuffer();
			if (isEOF)
				return -1;
		}

		int ret = (bufferlen >= len) ? len : bufferlen;

		System.arraycopy(buffer, bufferpos, b, off, ret);

		bufferpos += ret;
		bufferlen -= ret;

		return ret;
	}

	public int read(byte[] b) throws IOException
	{
		return read(b, 0, b.length);
	}

	public int read() throws IOException
	{
		if (bufferlen == 0)
		{
			fillBuffer();
			if (isEOF)
				return -1;
		}
		bufferlen--;
		return (buffer[bufferpos++] & 0xff);
	}
}
